@@ -19,7 +19,7 @@
 import math
 import operator
 from functools import reduce
-from typing import Callable
+from typing import Any, Callable
 
 import numpy as np
 import PIL
@@ -621,7 +621,7 @@ def filter_entropy(
     img: PIL.Image.Image,
     neighborhood: int = 9,
     threshold: float = 5.0,
-    relate: Callable[..., bool] = operator.gt,
+    relate: Callable[..., Any] = operator.gt,
 ) -> np.ndarray:
     """Filter image based on entropy (complexity).
 
@@ -677,7 +677,7 @@ def grays(img: PIL.Image.Image, tolerance: int = 15) -> np.ndarray:
     if np.array(img).ndim != 3:
         raise ValueError("Input must be 3D.")
     # TODO: class image mode exception: raise exception if not RGB(A)
-    img_arr = np.array(img).astype(np.int)
+    img_arr = np.array(img).astype(np.int64)
     rg_diff = abs(img_arr[:, :, 0] - img_arr[:, :, 1]) > tolerance
     rb_diff = abs(img_arr[:, :, 0] - img_arr[:, :, 2]) > tolerance
     gb_diff = abs(img_arr[:, :, 1] - img_arr[:, :, 2]) > tolerance
@@ -800,7 +800,7 @@ def hysteresis_threshold_mask(
 
 
 def otsu_threshold(
-    img: PIL.Image.Image, relate: Callable[..., bool] = operator.lt
+    img: PIL.Image.Image, relate: Callable[..., Any] = operator.lt
 ) -> np.ndarray:
     """Mask image based on pixel above Otsu threshold.
 
@@ -876,7 +876,7 @@ def red_filter(
 
 
 def yen_threshold(
-    img: PIL.Image.Image, relate: Callable[..., bool] = operator.lt
+    img: PIL.Image.Image, relate: Callable[..., Any] = operator.lt
 ) -> np.ndarray:
     """Mask image based on pixel below Yen's threshold.
 
