@@ -15,34 +15,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ------------------------------------------------------------------------
-
-"""Provides the Slide class.
-
-Slide is the main API class for manipulating slide objects.
-"""
-
 import math
+import ntpath
 import os
 import pathlib
-from functools import lru_cache
 from typing import Iterator, List, Tuple, Union
 
-import ntpath
 import numpy as np
 import openslide
 import PIL
+from skimage.measure import find_contours
 
 from .exceptions import LevelError
 from .filters.compositions import FiltersComposition
+from .masks import BinaryMask
 from .tile import Tile
-from .types import CoordinatePair, Region
-from .util import (
-    lazyproperty,
-    polygon_to_mask_array,
-    region_coordinates,
-    regions_from_binary_mask,
-    scale_coordinates,
-)
+from .types import CoordinatePair
+from .util import lazyproperty
 
 IMG_EXT = "png"
 
@@ -62,6 +51,9 @@ class Slide:
         self, path: Union[str, pathlib.Path], processed_path: Union[str, pathlib.Path]
     ) -> None:
         self._path = str(path) if isinstance(path, pathlib.Path) else path
+
+        if processed_path is None:
+            raise TypeError("processed_path cannot be None.")
         self._processed_path = processed_path
 
     def __repr__(self):
@@ -72,41 +64,20 @@ class Slide:
 
     # ---public interface methods and properties---
 
-    @lazyproperty
-    @lru_cache(maxsize=100)
-    def biggest_tissue_box_mask(self) -> np.ndarray:
-        """Return the thumbnail binary mask of the box containing the max tissue area.
-
-        Returns
-        -------
-        mask: np.ndarray
-            Binary mask of the box containing the max area of tissue. The dimensions are
-            those of the thumbnail.
-
-        """
-        thumb = self._wsi.get_thumbnail(self._thumbnail_size)
-        filters = FiltersComposition(Slide).tissue_mask_filters
-
-        thumb_mask = filters(thumb)
-        regions = regions_from_binary_mask(thumb_mask)
-        biggest_region = self._biggest_regions(regions, n=1)[0]
-        biggest_region_coordinates = region_coordinates(biggest_region)
-        thumb_bbox_mask = polygon_to_mask_array(
-            self._thumbnail_size, biggest_region_coordinates
-        )
-        return thumb_bbox_mask
-
     @lazyproperty
     def dimensions(self) -> Tuple[int, int]:
         """Return the slide dimensions (w,h) at level 0.
 
         Returns
         -------
-        dimensions : tuple(width, height)
+        dimensions : Tuple[int, int]
+            Slide dimensions (width, height)
         """
         return self._wsi.dimensions
 
-    def extract_tile(self, coords: CoordinatePair, level: int) -> Tile:
+    def extract_tile(
+        self, coords: CoordinatePair, level: int, tile_size: Tuple
+    ) -> Tile:
         """Extract a tile of the image at the selected level.
 
         Parameters
@@ -115,6 +86,8 @@ class Slide:
             Coordinates at level 0 from which to extract the tile.
         level : int
             Level from which to extract the tile.
+        tile_size: tuple
+            Final size of the tile (x,y).
 
         Returns
         -------
@@ -131,17 +104,8 @@ class Slide:
                 f"{self.dimensions}"
             )
 
-        coords_level = scale_coordinates(
-            reference_coords=coords,
-            reference_size=self.level_dimensions(level=0),
-            target_size=self.level_dimensions(level=level),
-        )
-
-        h_l = coords_level.y_br - coords_level.y_ul
-        w_l = coords_level.x_br - coords_level.x_ul
-
         image = self._wsi.read_region(
-            location=(coords.x_ul, coords.y_ul), level=level, size=(w_l, h_l)
+            location=(coords.x_ul, coords.y_ul), level=level, size=tile_size
         )
         tile = Tile(image, coords, level)
         return tile
@@ -156,7 +120,8 @@ class Slide:
 
         Returns
         -------
-        dimensions : tuple (width, height)
+        dimensions : Tuple[int, int]
+            Slide dimensions at the specified level (width, height)
         """
         level = level if level >= 0 else self._remap_level(level)
         try:
@@ -178,21 +143,24 @@ class Slide:
         """
         return list(range(len(self._wsi.level_dimensions)))
 
-    def locate_biggest_tissue_box(
+    def locate_mask(
         self,
+        binary_mask: BinaryMask,
         scale_factor: int = 32,
         tissue_mask: bool = False,
         alpha: int = 128,
         outline: str = "red",
     ) -> PIL.Image.Image:
-        """Draw biggest tissue box reference on a rescaled version of the slide
+        """Draw binary mask contours on a rescaled version of the slide
 
         Parameters
         ----------
+        binary_mask : BinaryMask
+            Binary Mask object
         scale_factor : int
             Scaling factor for the returned image. Default is 32.
         tissue_mask : bool, optional
-            Whether to draw the biggest tissue box on the binary tissue mask instead of
+            Whether to draw the contours on the binary tissue mask instead of
             the rescaled version of the slide. Default is False.
         alpha : int
             The alpha level to be applied to the rescaled slide, default to 128.
@@ -202,28 +170,31 @@ class Slide:
         Returns
         -------
         PIL.Image.Image
-            PIL Image of the rescaled slide with the biggest tissue bounding box
-            outlined.
+            PIL Image of the rescaled slide with the binary mask contours outlined.
         """
-        if not os.path.exists(self.scaled_image_path(scale_factor)):
-            self.save_scaled_image(scale_factor)
-        img = PIL.Image.open(self.scaled_image_path(scale_factor))
-
-        filters = FiltersComposition(Slide).tissue_mask_filters
-
-        img_tissue_mask = filters(img)
-        regions = regions_from_binary_mask(img_tissue_mask)
-        biggest_region = self._biggest_regions(regions, n=1)[0]
-
-        biggest_region_coordinates = region_coordinates(biggest_region)
+        img = self.scaled_image(scale_factor)
+        mask = binary_mask(self)
+        resized_mask = np.array(
+            PIL.Image.fromarray(mask).resize(img.size, PIL.Image.ANTIALIAS)
+        )
 
         if tissue_mask:
+            filters = FiltersComposition(Slide).tissue_mask_filters
+            img_tissue_mask = filters(img)
             img = PIL.Image.fromarray(img_tissue_mask).convert("RGB")
         else:
             img.putalpha(alpha)
 
-        draw = PIL.ImageDraw.Draw(img)
-        draw.rectangle(tuple(biggest_region_coordinates), outline=outline)
+        # pad the mask to have closed contours along the edges
+        padded_mask = np.pad(resized_mask, pad_width=1, mode="constant")
+        contours = [
+            cont - 1 for cont in find_contours(padded_mask, 0.5)
+        ]  # unpad countours
+
+        for contour in contours:
+            contour = np.ceil(contour)
+            contour = np.vstack([contour[:, 1], contour[:, 0]]).T
+            PIL.ImageDraw.Draw(img).polygon(contour.ravel().tolist(), outline=outline)
 
         return img
 
@@ -266,6 +237,7 @@ class Slide:
         ----------
         scale_factor : int, optional
             Image scaling factor. Default is 32.
+
         Returns
         ----------
         resampled_array: np.ndarray
@@ -273,30 +245,8 @@ class Slide:
         """
         return self._resample(scale_factor)[1]
 
-    def save_scaled_image(self, scale_factor: int = 32) -> None:
-        """Save a scaled image in the correct path
-
-        Parameters
-        ----------
-        scale_factor : int, optional
-            Image scaling factor. Default is 32.
-        """
-        os.makedirs(self._processed_path, exist_ok=True)
-        img = self._resample(scale_factor)[0]
-        img.save(self.scaled_image_path(scale_factor))
-
-    def save_thumbnail(self) -> None:
-        """Save a thumbnail in the correct path"""
-        os.makedirs(self._processed_path, exist_ok=True)
-
-        img = self._wsi.get_thumbnail(self._thumbnail_size)
-
-        folder = os.path.dirname(self.thumbnail_path)
-        pathlib.Path(folder).mkdir(exist_ok=True)
-        img.save(self.thumbnail_path)
-
-    def scaled_image_path(self, scale_factor: int = 32) -> str:
-        """Return slide image path.
+    def scaled_image(self, scale_factor: int = 32) -> PIL.Image.Image:
+        """Return a scaled image of the slide.
 
         Parameters
         ----------
@@ -305,10 +255,10 @@ class Slide:
 
         Returns
         -------
-        img_path : str
+        PIL.Image.Image
+            A scaled image of the slide.
         """
-        img_path = self._breadcrumb(self._processed_path, scale_factor)
-        return img_path
+        return self._resample(scale_factor)[0]
 
     def show(self) -> None:
         """Display the slide thumbnail.
@@ -316,80 +266,24 @@ class Slide:
         NOTE: A new window of your OS image viewer will be opened.
         """
         try:
-            thumbnail = PIL.Image.open(self.thumbnail_path)
+            thumbnail = self.thumbnail
             thumbnail.show()  # pragma: no cover
         except FileNotFoundError as error:
-            raise FileNotFoundError(f"Cannot display the slide thumbnail:{error}")
+            raise FileNotFoundError(f"Cannot display the slide thumbnail: {error}")
 
     @lazyproperty
-    def thumbnail_path(self) -> str:
-        """Return thumbnail image path.
+    def thumbnail(self) -> PIL.Image.Image:
+        """Return the slide thumbnail.
 
         Returns
         -------
-        thumb_path : str
+        PIL.Image.Image
+            The slide thumbnail.
         """
-        thumb_path = os.path.join(
-            self._processed_path, "thumbnails", f"{self.name}.{IMG_EXT}"
-        )
-        return thumb_path
+        return self._wsi.get_thumbnail(self._thumbnail_size)
 
     # ------- implementation helpers -------
 
-    @staticmethod
-    def _biggest_regions(regions: List[Region], n: int = 1) -> List[Region]:
-        """Return the biggest ``n`` regions.
-
-        Parameters
-        ----------
-        regions : List[Region]
-            List of regions
-        n : int, optional
-            Number of regions to return, by default 1
-
-        Returns
-        -------
-        List[Region]
-            List of ``n`` biggest regions
-
-        Raises
-        ------
-        ValueError
-            If ``n`` is not between 1 and the number of elements of ``regions``
-        """
-
-        if not 1 <= n <= len(regions):
-            raise ValueError(f"n should be between 1 and {len(regions)}, got {n}")
-
-        sorted_regions = sorted(regions, key=lambda r: r.area, reverse=True)
-        return sorted_regions[:n]
-
-    def _breadcrumb(self, directory_path: str, scale_factor: int = 32) -> str:
-        """Returns a complete path according to the give directory path
-
-        Parameters
-        ----------
-        directory_path: str
-        scale_factor : int, 32 by default
-            Image scaling factor
-
-        Returns
-        -------
-        final_path: str, a real and complete path starting from the dir path
-                    e.g. /processed_path/my-image-name-x32/
-                         /thumb_path/my-image-name-x32/thumbs
-        """
-        large_w, large_h, new_w, new_h = self._resampled_dimensions(scale_factor)
-        if {large_w, large_h, new_w, new_h} == {None}:
-            final_path = os.path.join(directory_path, f"{self.name}*.{IMG_EXT}")
-        else:
-            final_path = os.path.join(
-                directory_path,
-                f"{self.name}-{scale_factor}x-{large_w}x{large_h}-{new_w}x"
-                f"{new_h}.{IMG_EXT}",
-            )
-        return final_path
-
     def _has_valid_coords(self, coords: CoordinatePair) -> bool:
         """Check if ``coords`` are valid 0-level coordinates.
 
@@ -416,17 +310,17 @@ class Slide:
         Parameters
         ----------
         level : int
-            the level index to remap
+            The level index to remap
+
+        Returns
+        -------
+        level : int
+           Positive level index
 
         Raises
         ------
         LevelError
             when the abs(level) is greater than the number of the levels.
-
-        Returns
-        -------
-        level : int
-           positive level index
         """
         if len(self.levels) - abs(level) < 0:
             raise LevelError(
@@ -436,7 +330,7 @@ class Slide:
         return len(self.levels) - abs(level)
 
     def _resample(self, scale_factor: int = 32) -> Tuple[PIL.Image.Image, np.array]:
-        """Converts a slide to a scaled-down PIL image.
+        """Convert a slide to a scaled-down PIL image.
 
         The PIL image is also converted to array.
         image is the scaled-down PIL image, original width and original height
@@ -450,7 +344,10 @@ class Slide:
 
         Returns
         -------
-        img, arr_img
+        PIL.Image.Image
+            The resampled image
+        np.ndarray
+            The resampled image converted to array
         """
 
         _, _, new_w, new_h = self._resampled_dimensions(scale_factor)
@@ -476,8 +373,9 @@ class Slide:
 
         Returns
         -------
-        tuple
-            Original slide dimensions and scaled dimensions
+        Tuple[int, int, int, int]
+            Original slide dimensions and scaled dimensions (original w, original h,
+            resampled w, resampled h).
         """
         large_w, large_h = self.dimensions
         new_w = math.floor(large_w / scale_factor)
@@ -485,7 +383,7 @@ class Slide:
         return large_w, large_h, new_w, new_h
 
     @lazyproperty
-    def _thumbnail_size(self) -> Tuple:
+    def _thumbnail_size(self) -> Tuple[int, int]:
         r"""Compute the thumbnail size proportionally to the slide dimensions.
 
         If the size of the slide is (v, m) where v has magnitude w and m has magnitude
@@ -509,7 +407,7 @@ class Slide:
 
         Returns
         -------
-        Tuple
+        Tuple[int, int]
             Thumbnail size
         """
         return tuple(
@@ -535,7 +433,9 @@ class Slide:
                 "Your wsi has something broken inside, a doctor is needed"
             )
         except FileNotFoundError:
-            raise FileNotFoundError("The wsi path resource doesn't exist")
+            raise FileNotFoundError(
+                f"The wsi path resource doesn't exist: {self._path}"
+            )
         return slide
 
 
@@ -580,39 +480,56 @@ class SlideSet:
 
     # ---public interface methods and properties---
 
-    def save_scaled_slides(self, scale_factor: int = 32, n: int = 0) -> None:
-        """Save rescaled images
+    def scaled_images(
+        self, scale_factor: int = 32, n: int = 0
+    ) -> List[PIL.Image.Image]:
+        """Return rescaled images of the slides.
 
         Parameters
         ----------
-        scale_factor: int, optional
+        scale_factor : int, optional
             Image scaling factor. Default is 32.
-        n: int, optional
-            First n slides in dataset folder to rescale and save. Default is 0, meaning
-            that all the slides will be saved.
-        """
-        os.makedirs(self._processed_path, exist_ok=True)
-        n = self.total_slides if (n > self.total_slides or n == 0) else n
-        for slide in list(self.__iter__())[:n]:
-            slide.save_scaled_image(scale_factor)
+        n : int, optional
+            First n slides in dataset folder to rescale. Default is 0, meaning that all
+            the slides will be returned.
 
-    def save_thumbnails(self, n: int = 0) -> None:
-        """Save thumbnails
+        Returns
+        -------
+        List[PIL.Image.Image]
+            List of rescaled images of the slides.
+        """
+        n = self.total_slides if (n > self.total_slides or n == 0) else n
+        rescaled_imgs = []
+        for slide in list(self.__iter__())[:n]:
+            rescaled_imgs.append(slide.scaled_image(scale_factor))
+        return rescaled_imgs
+
+    def thumbnails(self, n: int = 0) -> List[PIL.Image.Image]:
+        """Return slides thumbnails
 
         Parameters
         ----------
-        n: int. optional
+        n : int, optional
             First n slides in dataset folder. Default is 0, meaning that the thumbnails
-            of all the slides will be saved.
+            of all the slides will be returned.
+
+        Returns
+        -------
+        List[PIL.Image.Image]
+            List of slides thumbnails
         """
-        os.makedirs(self._processed_path, exist_ok=True)
         n = self.total_slides if (n > self.total_slides or n == 0) else n
+        thumbnails = []
         for slide in list(self.__iter__())[:n]:
-            slide.save_thumbnail()
+            thumbnails.append(slide.thumbnail)
+        return thumbnails
 
     @lazyproperty
     def slides_stats(self) -> dict:
-        """Retrieve statistic/graphs of slides files contained in the dataset.
+        """Compute statistics for the WSI collection, namely the number of available
+        slides; the slide with the maximum/minimum width; the slide with the
+        maximum/minimum height; the slide with the maximum/minimum size; the average
+        width/height/size of the slides.
 
         Returns
         ----------
