@@ -20,18 +20,19 @@ import csv
 import logging
 import os
 from abc import abstractmethod
-from functools import lru_cache
 from typing import List, Tuple
 
 import numpy as np
 import PIL
 
 from .exceptions import LevelError, TileSizeError
+from .masks import BiggestTissueBoxMask, BinaryMask
 from .scorer import Scorer
 from .slide import Slide
 from .tile import Tile
 from .types import CoordinatePair
 from .util import (
+    random_choice_true_mask2d,
     region_coordinates,
     regions_from_binary_mask,
     scale_coordinates,
@@ -51,33 +52,21 @@ class Tiler(Protocol):
     """General tiler object"""
 
     level: int
-    tile_size: int
-
-    @lru_cache(maxsize=100)
-    def box_mask(self, slide: Slide) -> np.ndarray:
-        """Return binary mask, at thumbnail level, of the box for tiles extraction.
-
-        The mask pixels set to True correspond to the tissue box.
-
-        Parameters
-        ----------
-        slide : Slide
-            The Slide from which to extract the extraction mask
-
-        Returns
-        -------
-        np.ndarray
-            Extraction mask at thumbnail level
-        """
-        return slide.biggest_tissue_box_mask
+    tile_size: Tuple[int, int]
 
     @abstractmethod
-    def extract(self, slide: Slide, log_level: str):
-        raise NotImplementedError
+    def extract(
+        self,
+        slide: Slide,
+        log_level: str,
+        extraction_mask: BinaryMask = BiggestTissueBoxMask(),
+    ):
+        pass  # pragma: no cover
 
     def locate_tiles(
         self,
         slide: Slide,
+        extraction_mask: BinaryMask = BiggestTissueBoxMask(),
         scale_factor: int = 32,
         alpha: int = 128,
         outline: str = "red",
@@ -88,6 +77,9 @@ class Tiler(Protocol):
         ----------
         slide : Slide
             Slide reference where placing the tiles
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`
         scale_factor: int
             Scaling factor for the returned image. Default is 32.
         alpha: int
@@ -100,16 +92,14 @@ class Tiler(Protocol):
         PIL.Image.Image
             PIL Image of the rescaled slide with the extracted tiles outlined
         """
-        if not os.path.exists(slide.scaled_image_path(scale_factor)):
-            slide.save_scaled_image(scale_factor)
-        img = PIL.Image.open(slide.scaled_image_path(scale_factor))
+        img = slide.scaled_image(scale_factor)
         img.putalpha(alpha)
         draw = PIL.ImageDraw.Draw(img)
 
         tiles = (
-            self._tiles_generator(slide)[0]
+            self._tiles_generator(slide, extraction_mask)[0]
             if isinstance(self, ScoreTiler)
-            else self._tiles_generator(slide)
+            else self._tiles_generator(slide, extraction_mask)
         )
         tiles_coords = (tile[1] for tile in tiles)
         for coords in tiles_coords:
@@ -165,8 +155,10 @@ class Tiler(Protocol):
 
         return tile_filename
 
-    def _tiles_generator(self, slide: Slide) -> None:
-        raise NotImplementedError
+    def _tiles_generator(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> Tuple[Tile, CoordinatePair]:
+        pass  # pragma: no cover
 
     def _validate_level(self, slide: Slide) -> None:
         """Validate the Tiler's level according to the Slide.
@@ -250,7 +242,12 @@ class GridTiler(Tiler):
         self.prefix = prefix
         self.suffix = suffix
 
-    def extract(self, slide: Slide, log_level: str = "INFO") -> None:
+    def extract(
+        self,
+        slide: Slide,
+        extraction_mask: BinaryMask = BiggestTissueBoxMask(),
+        log_level: str = "INFO",
+    ) -> None:
         """Extract tiles arranged in a grid and save them to disk, following this
         filename pattern:
         `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}-{y_br_wsi}{suffix}`
@@ -259,7 +256,10 @@ class GridTiler(Tiler):
         ----------
         slide : Slide
             Slide from which to extract the tiles
-        log_level: str, {"DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"}
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
+        log_level : str, {"DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"}
             Threshold level for the log messages. Default "INFO"
 
         Raises
@@ -274,17 +274,18 @@ class GridTiler(Tiler):
         self._validate_level(slide)
         self._validate_tile_size(slide)
 
-        grid_tiles = self._tiles_generator(slide)
+        grid_tiles = self._tiles_generator(slide, extraction_mask)
         tiles_counter = 0
         for tiles_counter, (tile, tile_wsi_coords) in enumerate(grid_tiles):
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
-            full_tile_path = os.path.join(slide.processed_path, "tiles", tile_filename)
+            full_tile_path = os.path.join(slide.processed_path, tile_filename)
             tile.save(full_tile_path)
-            logging.info(f"\t Tile {tiles_counter} saved: {tile_filename}")
-        logging.info(f"{tiles_counter} Grid Tiles have been saved.")
+            logger.info(f"\t Tile {tiles_counter} saved: {tile_filename}")
+        logger.info(f"{tiles_counter} Grid Tiles have been saved.")
 
     @property
     def tile_size(self) -> Tuple[int, int]:
+        """(width, height) of the extracted tiles."""
         return self._valid_tile_size
 
     @tile_size.setter
@@ -337,7 +338,9 @@ class GridTiler(Tiler):
                 )
                 yield tile_wsi_coords
 
-    def _grid_coordinates_generator(self, slide: Slide) -> CoordinatePair:
+    def _grid_coordinates_generator(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> CoordinatePair:
         """Generate Coordinates at level 0 of grid tiles within the tissue.
 
         Parameters
@@ -345,34 +348,42 @@ class GridTiler(Tiler):
         slide : Slide
             Slide from which to calculate the coordinates. Needed to calculate the
             tissue area.
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Yields
         -------
         Iterator[CoordinatePair]
             Iterator of tiles' CoordinatePair
         """
-        box_mask = self.box_mask(slide)
+        binary_mask = extraction_mask(slide)
 
-        regions = regions_from_binary_mask(box_mask)
+        regions = regions_from_binary_mask(binary_mask)
         # ----at the moment there is only one region----
         for region in regions:
             bbox_coordinates_thumb = region_coordinates(region)
             bbox_coordinates = scale_coordinates(
                 bbox_coordinates_thumb,
-                box_mask.shape[::-1],
+                binary_mask.shape[::-1],
                 slide.level_dimensions(self.level),
             )
             yield from self._grid_coordinates_from_bbox_coordinates(
                 bbox_coordinates, slide
             )
 
-    def _tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
+    def _tiles_generator(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> Tuple[Tile, CoordinatePair]:
         """Generator of tiles arranged in a grid.
 
         Parameters
         ----------
         slide : Slide
             Slide from which to extract the tiles
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Yields
         -------
@@ -381,10 +392,12 @@ class GridTiler(Tiler):
         CoordinatePair
             Coordinates of the slide at level 0 from which the tile has been extracted
         """
-        grid_coordinates_generator = self._grid_coordinates_generator(slide)
+        grid_coordinates_generator = self._grid_coordinates_generator(
+            slide, extraction_mask
+        )
         for coords in grid_coordinates_generator:
             try:
-                tile = slide.extract_tile(coords, self.level)
+                tile = slide.extract_tile(coords, self.level, self.tile_size)
             except ValueError:
                 continue
 
@@ -480,7 +493,12 @@ class RandomTiler(Tiler):
         self.prefix = prefix
         self.suffix = suffix
 
-    def extract(self, slide: Slide, log_level: str = "INFO") -> None:
+    def extract(
+        self,
+        slide: Slide,
+        extraction_mask: BinaryMask = BiggestTissueBoxMask(),
+        log_level: str = "INFO",
+    ) -> None:
         """Extract random tiles and save them to disk, following this filename pattern:
         `{prefix}tile_{tiles_counter}_level{level}_{x_ul_wsi}-{y_ul_wsi}-{x_br_wsi}-{y_br_wsi}{suffix}`
 
@@ -488,6 +506,9 @@ class RandomTiler(Tiler):
         ----------
         slide : Slide
             Slide from which to extract the tiles
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
         log_level: str, {"DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"}
             Threshold level for the log messages. Default "INFO"
 
@@ -503,15 +524,15 @@ class RandomTiler(Tiler):
         self._validate_level(slide)
         self._validate_tile_size(slide)
 
-        random_tiles = self._tiles_generator(slide)
+        random_tiles = self._tiles_generator(slide, extraction_mask)
 
         tiles_counter = 0
         for tiles_counter, (tile, tile_wsi_coords) in enumerate(random_tiles):
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
-            full_tile_path = os.path.join(slide.processed_path, "tiles", tile_filename)
+            full_tile_path = os.path.join(slide.processed_path, tile_filename)
             tile.save(full_tile_path)
-            logging.info(f"\t Tile {tiles_counter} saved: {tile_filename}")
-        logging.info(f"{tiles_counter+1} Random Tiles have been saved.")
+            logger.info(f"\t Tile {tiles_counter} saved: {tile_filename}")
+        logger.info(f"{tiles_counter+1} Random Tiles have been saved.")
 
     @property
     def max_iter(self) -> int:
@@ -538,45 +559,51 @@ class RandomTiler(Tiler):
 
     # ------- implementation helpers -------
 
-    def _random_tile_coordinates(self, slide: Slide) -> CoordinatePair:
+    def _random_tile_coordinates(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> CoordinatePair:
         """Return 0-level Coordinates of a tile picked at random within the box.
 
         Parameters
         ----------
         slide : Slide
             Slide from which calculate the coordinates. Needed to calculate the box.
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Returns
         -------
         CoordinatePair
             Random tile Coordinates at level 0
         """
-        box_mask = self.box_mask(slide)
+        binary_mask = extraction_mask(slide)
         tile_w_lvl, tile_h_lvl = self.tile_size
 
-        x_ul_lvl = np.random.choice(np.where(box_mask)[1])
-        y_ul_lvl = np.random.choice(np.where(box_mask)[0])
+        x_ul_lvl, y_ul_lvl = random_choice_true_mask2d(binary_mask)
 
-        # Scale tile dimensions to thumbnail dimensions
+        # Scale tile dimensions to extraction mask dimensions
         tile_w_thumb = (
-            tile_w_lvl * box_mask.shape[1] / slide.level_dimensions(self.level)[0]
+            tile_w_lvl * binary_mask.shape[1] / slide.level_dimensions(self.level)[0]
         )
         tile_h_thumb = (
-            tile_h_lvl * box_mask.shape[0] / slide.level_dimensions(self.level)[1]
+            tile_h_lvl * binary_mask.shape[0] / slide.level_dimensions(self.level)[1]
         )
 
         x_br_lvl = x_ul_lvl + tile_w_thumb
         y_br_lvl = y_ul_lvl + tile_h_thumb
 
         tile_wsi_coords = scale_coordinates(
-            reference_coords=CoordinatePair(x_ul_lvl, y_ul_lvl, x_br_lvl, y_br_lvl),
-            reference_size=box_mask.shape[::-1],
+            reference_coords=CoordinatePair(y_ul_lvl, x_ul_lvl, y_br_lvl, x_br_lvl),
+            reference_size=binary_mask.shape[::-1],
             target_size=slide.dimensions,
         )
 
         return tile_wsi_coords
 
-    def _tiles_generator(self, slide: Slide) -> Tuple[Tile, CoordinatePair]:
+    def _tiles_generator(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> Tuple[Tile, CoordinatePair]:
         """Generate Random Tiles within a slide box.
 
         Stops if:
@@ -587,6 +614,9 @@ class RandomTiler(Tiler):
         ----------
         slide : Slide
             The Whole Slide Image from which to extract the tiles.
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Yields
         ------
@@ -599,9 +629,9 @@ class RandomTiler(Tiler):
         iteration = valid_tile_counter = 0
 
         while True:
-            tile_wsi_coords = self._random_tile_coordinates(slide)
+            tile_wsi_coords = self._random_tile_coordinates(slide, extraction_mask)
             try:
-                tile = slide.extract_tile(tile_wsi_coords, self.level)
+                tile = slide.extract_tile(tile_wsi_coords, self.level, self.tile_size)
             except ValueError:
                 iteration -= 1
                 continue
@@ -677,7 +707,11 @@ class ScoreTiler(GridTiler):
         )
 
     def extract(
-        self, slide: Slide, report_path: str = None, log_level: str = "INFO"
+        self,
+        slide: Slide,
+        extraction_mask: BinaryMask = BiggestTissueBoxMask(),
+        report_path: str = None,
+        log_level: str = "INFO",
     ) -> None:
         """Extract grid tiles and save them to disk, according to a scoring function and
         following this filename pattern:
@@ -689,6 +723,9 @@ class ScoreTiler(GridTiler):
         ----------
         slide : Slide
             Slide from which to extract the tiles
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
         report_path : str, optional
             Path to the CSV report. If None, no report will be saved
         log_level: str, {"DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"}
@@ -706,17 +743,19 @@ class ScoreTiler(GridTiler):
         self._validate_level(slide)
         self._validate_tile_size(slide)
 
-        highest_score_tiles, highest_scaled_score_tiles = self._tiles_generator(slide)
+        highest_score_tiles, highest_scaled_score_tiles = self._tiles_generator(
+            slide, extraction_mask
+        )
 
         tiles_counter = 0
         filenames = []
 
         for tiles_counter, (score, tile_wsi_coords) in enumerate(highest_score_tiles):
-            tile = slide.extract_tile(tile_wsi_coords, self.level)
+            tile = slide.extract_tile(tile_wsi_coords, self.level, self.tile_size)
             tile_filename = self._tile_filename(tile_wsi_coords, tiles_counter)
-            tile.save(os.path.join(slide.processed_path, "tiles", tile_filename))
+            tile.save(os.path.join(slide.processed_path, tile_filename))
             filenames.append(tile_filename)
-            logging.info(
+            logger.info(
                 f"\t Tile {tiles_counter} - score: {score} saved: {tile_filename}"
             )
 
@@ -725,12 +764,12 @@ class ScoreTiler(GridTiler):
                 report_path, highest_score_tiles, highest_scaled_score_tiles, filenames
             )
 
-        logging.info(f"{tiles_counter+1} Grid Tiles have been saved.")
+        logger.info(f"{tiles_counter+1} Grid Tiles have been saved.")
 
     # ------- implementation helpers -------
 
     def _tiles_generator(
-        self, slide: Slide
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
     ) -> Tuple[List[Tuple[float, CoordinatePair]], List[Tuple[float, CoordinatePair]]]:
         r"""Calculate the tiles with the highest scores and their extraction coordinates
 
@@ -738,6 +777,9 @@ class ScoreTiler(GridTiler):
         ----------
         slide : Slide
             The slide to extract the tiles from.
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Returns
         -------
@@ -759,7 +801,7 @@ class ScoreTiler(GridTiler):
         ValueError
             If ``n_tiles`` is negative.
         """  # noqa
-        all_scores = self._scores(slide)
+        all_scores = self._scores(slide, extraction_mask)
         scaled_scores = self._scale_scores(all_scores)
 
         sorted_tiles_by_score = sorted(all_scores, key=lambda x: x[0], reverse=True)
@@ -846,13 +888,18 @@ class ScoreTiler(GridTiler):
 
         return list(zip(scores_scaled, coords))
 
-    def _scores(self, slide: Slide) -> List[Tuple[float, CoordinatePair]]:
+    def _scores(
+        self, slide: Slide, extraction_mask: BinaryMask = BiggestTissueBoxMask()
+    ) -> List[Tuple[float, CoordinatePair]]:
         """Calculate the scores for all the tiles extracted from the ``slide``.
 
         Parameters
         ----------
         slide : Slide
             The slide to extract the tiles from.
+        extraction_mask : BinaryMask, optional
+            BinaryMask object defining how to compute a binary mask from a Slide.
+            Default `BiggestTissueBoxMask`.
 
         Returns
         -------
@@ -860,12 +907,12 @@ class ScoreTiler(GridTiler):
             List of tuples containing the score and the extraction coordinates for each
             tile. Each tuple represents a tile.
         """
-        if next(super()._tiles_generator(slide), None) is None:
+        if next(super()._tiles_generator(slide, extraction_mask), None) is None:
             raise RuntimeError(
                 "No tiles have been generated. This could happen if `check_tissue=True`"
             )
 
-        grid_tiles = super()._tiles_generator(slide)
+        grid_tiles = super()._tiles_generator(slide, extraction_mask)
         scores = []
 
         for tile, tile_wsi_coords in grid_tiles:
